%SCRIPT FOR CALCULATING COEFFICIENTS FOR THE FOLLOWING FILTER
%
% Lanczos2-windowed sinc function
% 5 taps
% 16 phases per tap
%
% Author: ZIPcores.com (2009)
% "Estimating polyphase filter coefficients with a windowed function"
% 

% Calculate coefficients for Phases 0 to 15,
% Taps 0,1,2,3,4
for p_index = 1:16
    for t_index = 1:5
        p = (p_index - 1)/16;
        t = (t_index - 1);
        x = t - 2 - p;
        coeff(p_index, t_index) = sinc(x) * sinc(x/2);
    end
end

% Quantize to 2.6-bit signed numbers
coeff_quant = round(coeff * 64);

% Check they sum to 1
sum_coeff = sum(coeff,2);
sum_coeff_quant = sum(coeff_quant,2);

% Write coefficients to a file I=Phase, J=Tap
fid = fopen('coeffs_5tap.txt', 'w');
fprintf(fid, ' TAP0 TAP1 TAP2 TAP3 TAP4 SUM\n\n');
for I = 1:16
    fprintf(fid, 'PHASE%2d : ',I-1 );
for J = 1:5
    fprintf(fid, '%5d ', coeff_quant(I,J));
end
    if sum_coeff_quant(I) == 64
        fprintf(fid, '%5d\n', sum_coeff_quant(I));
    else
      fprintf(fid, '%5d *\n', sum_coeff_quant(I));
    end
end

% The script stores the calculated coefficents to a text file.
% The values labelled with a asterisk (*) in the output text file must be
% adjusted in order to obtain sum to unity.
%