function pixel = winscale_4pix( pix, cX, cY, winW, winH,sourW,sourH, SF, winX, winY, scaletype)
%WINSCALE_4PIX 
% 4PIX  -   a 2x2 pixel frame from original image
% WINW  - window incremental horizontal ([W]idth) value
% WINH  - window incremental vertical ([H]eight) value
% SOURW - source incremental horizontal ([W]idth) value
% SOURH - source incremental vertical ([H]eight) value
% SF    - scaling factor
% WINX  - filter center x-coordinate
% WINY  - filter center y-coordinate
% CX    - C0 x-coordinate
% CY    - C0 y-coordinate
% SCALETYPE - 'upscaling' or 'downscaling'

%--------------------------------------------------
% COEFFICIENT AND AREA CALCULATIONS
%
%   dL = winX - winLX
%   dR = 0.0
%   dT = winY - winTY
%   dB = 0.0
%
%   A0 = dL * dT
%   A1 = dL * (winH - dT)
%   A2 = dT * (winw - dL)
%   A3 = (winH - dT) * (winW - dL)
%
%   W0 = A0 * SF
%   W1 = A1 * SF
%   W2 = A2 * SF
%   W3 = 1.0 - W0 - W1 - W2

C0 = pix(1,1);
C1 = pix(2,1);
C2 = pix(1,2);
C3 = pix(2,2);


C0BX = cX - sourW/2;
C0BY = cY - sourH/2;
C3BX = C0BX + sourW;
C3BY = C0BY + sourH;
PBX = winX - winW/2;
PBY = winY - winH/2;

% Calculate delta overlapping distances

if strcmp('upscaling',scaletype)
    
    dL = winscale_getDelta(winW,winX,cX);
    dT = winscale_getDelta(winH,winY,cY);

else
    
    dL = (C0BX + sourW) - PBX;
    dT = (C0BY + sourH) - PBY;
    

end

    A0 = dL * dT;
    A1 = dL * (winH - dT);
    A2 = dT * (winW - dL);
    A3 = (winH - dT) * (winW - dL);

W0 = A0 * SF;
W1 = A1 * SF;
W2 = A2 * SF;
W3 = 1.0 - W0 - W1 - W2;


pixel = (W0 * C0) + (W1 * C1) + (W2 * C2) + (W3 * C3);


end

