function target_image = winscale_top( source_image, newHeight, newWidth, type)
%WINSCALE_TOP 
%   SOURCE_IMAGE    - original image ( oldHeight x oldWidth) [IMG]
%   NEWHEIGHT       - scaled height [PIX]
%   NEWWIDTH        - scaled width [PIX]
%   TARGET_IMAGE    - scaled image (newHeight x newWidth) [IMG]
%   TYPE            - for testing 'int' or 'float'

%--------------------------------------------------
% SOURCE IMAGE
%
%   C0  C2
%
%   C1  C3
%
%
%   C0-C3 are pixelvalues
%   oldWidth -
%   oldHeight -
%

oldWidth = size(source_image,2);
oldHeight = size(source_image,1);

%--------------------------------------------------
% TARGET IMAGE
%
%   A0  A2
%
%   A1  A3
%
%   
%   A0 - A3 are size of the overlapping areas

%output_image = zeros(newHeight,newWidth);
output_image = zeros(newHeight,newWidth);


%--------------------------------------------------
% DISPLAY SECTION
disp('-----------------------');
disp('- NEW SCALING STARTED ')
disp(sprintf('-- oldWidth : %f',oldWidth));
disp(sprintf('-- oldHeight : %f',oldHeight));
disp('--')
disp(sprintf('-- newWidth : %f',newWidth));
disp(sprintf('-- newHeight : %f',newHeight));
disp('--')



if (oldWidth > newWidth) && (oldHeight > newHeight)
    %-----------------------------------------------------
    % DOWNSCALING
    disp('-----  DOWNSCALING -----');
    
    scaletype = 'downscaling';
    
    SF = 1.0;
    
    winW = 1.0;
    winH = 1.0;
    
    sourW = newWidth/oldWidth;
    sourH = newHeight/oldHeight;

%-------------------------------------------------
% Calculating coordinates for old pixels. (TARGET INDEXED)
% Same as "Coordinate Accumulator" in article
next_x = zeros(1,newWidth);
next_x(1) = 0.5 + sourW/2;
for i=1:(oldWidth-1)
    next_x(i+1) = next_x(i) + sourW;
end

next_y = zeros(1,newHeight);
next_y(1) = 0.5 + sourH/2;
for i=1:(oldHeight-1)
    next_y(i+1) = next_y(i) + sourH;
end

%next_x;
%next_y;

          %Initial value for source pixels are allways source(1,1) for
          %downscaling
          source_x= 1;
          %source_y = 1;
    
    for target_x=1:newWidth
        source_y = 1;
        for target_y=1:newHeight
        
        %upper_border = next_y(source_y) - sourH/2;
        lower_border = next_y(source_y)+ sourH/2;
        %left_border = next_x(source_x) - sourW/2;
        right_border = next_x(source_x) + sourW/2;  
          
         
        upper_window_border = target_y - winH/2; 
        left_window_border = target_x - winW/2;
       
        
        %Check if window border has moved so much that source pixels has
        %changed in y-direction
        while upper_window_border >= lower_border
            source_y = source_y + 1;
            lower_border = next_y(source_y)+ sourH/2;
        end
        
        %Check if window border has moved so much that source pixels has
        %changed in x-direction
        while left_window_border >= right_border
            source_x = source_x + 1;
            right_border = next_x(source_x) + sourW/2; 
        end
        
        
        
        %source_y
        %source_x
        
        
        
        source_4pix = zeros(2,2);
        source_4pix(1,1) = source_image(source_y     , source_x);     %C0
        source_4pix(2,1) = source_image(source_y + 1 , source_x); %C1
        source_4pix(1,2) = source_image(source_y     , source_x +1 );     %C2
        source_4pix(2,2) = source_image(source_y + 1 , source_x + 1); %C3
        
        

        
        if strcmp('int',type)
             output_image(target_y,target_x) = floor(winscale_4pix(source_4pix,next_x(source_x),next_y(source_y),winW, winH,sourW,sourH,SF,target_x,target_y,scaletype));
           else
             if strcmp('float',type)
                output_image(target_y,target_x) = winscale_4pix(source_4pix,next_x(source_x),next_y(source_y),winW, winH,sourW,sourH,SF,target_x,target_y,scaletype);
             else
                disp('No TYPE (int or float) assigned');
             end
           
           end
        
        end
    end
    
    
    
    
    


    
else
    if (oldWidth < newWidth) && (oldHeight < newHeight)
    %-----------------------------------------------------
    % UPSCALING
    disp('------  UPSCALING ------');

    scaletype = 'upscaling';
    
    sourW = 1.0;
    sourH = 1.0;

%--------------------------------------------------
% FILTER WINDOW
%
%   winX,winY - center coordinates
%
%
%   winW = (1/HSR) - incremental horizontal ([W]idth) value
%   winH = (1/VSR) - incremental vertical ([H]eight) value
%
%   HSR = newWidth/oldWidth - [H]orizontal [S]caling [R]atio
%   VSR = newHeight/oldHeight - [V]ertical [S]caling [R]atio
%
%   SF = 1.0/(winW*winH) 
%

HSR = newWidth/oldWidth;
VSR = newHeight/oldHeight;

winW = (1/HSR);
winH = (1/VSR);

SF = 1.0/(winW*winH);


%--------------------------------------------------
% DISPLAY SECTION
disp(sprintf('-- horizontal scaling factor HSR : %f',HSR));
disp(sprintf('-- vertical scaling factor VSR : %f',VSR));
disp('--')
disp(sprintf('-- SF : %f',SF));



%-------------------------------------------------
% Calculating coordinates for new pixels.
% Same as "Coordinate Accumulator" in article
next_x = zeros(1,newWidth);
next_x(1) = 0.5 + winW/2;
for i=1:(newWidth-1)
    next_x(i+1) = next_x(i) + winW;
end

next_y = zeros(1,newHeight);
next_y(1) = 0.5 + winH/2;
for i=1:(newHeight-1)
    next_y(i+1) = next_y(i) + winH;
end

%next_x
%next_y



%target_x = 1;
%target_y = 4;

%source_image

for target_x=1:newWidth
    for target_y=1:newHeight
        upper_border = next_y(target_y) - winH/2;
        lower_border = next_y(target_y)+ winH/2;
        left_border = next_x(target_x) - winW/2;
        right_border = next_x(target_x) + winW/2;
            
        upper_source_row = round(upper_border);
        
        
        left_source_row = round(left_border);
        %disp(sprintf('uppser_source_row = %f',upper_source_row));
        %disp(sprintf('left_source_row = %f',left_source_row));
        % Border conditions
        %lower_border >= oldHeight + 0.5
        
       %((oldHeight + 0.5) - lower_border)
        
       %Check if current frame is a border frame on lower boundry
        if(((oldHeight - 0.5) - upper_border)<(10^-10))
            upper_source_row = upper_source_row -1;
         %   disp(sprintf('Lower boundry reached. upper_source_row changed = %f',upper_source_row))
        end
        
        %Check if current target frame is a border frame right boundry
        if(((oldWidth - 0.5) - left_border)<(10^-10))
            left_source_row = left_source_row - 1;
        end
        
        
        %disp(sprintf('left_source_row = %f',left_source_row));
        %disp(sprintf('upper_source_row = %f',upper_source_row));
        

        %Extracting the 4 pixels overlapped by the window
        source_4pix = zeros(2,2);
        source_4pix(1,1) = source_image(upper_source_row     , left_source_row);     %C0
        source_4pix(2,1) = source_image(upper_source_row + 1 , left_source_row); %C1
        source_4pix(1,2) = source_image(upper_source_row     , left_source_row +1 );     %C2
        source_4pix(2,2) = source_image(upper_source_row + 1 , left_source_row + 1); %C3
        
        
        %source_4pix;
        
        %Calculate interpolated value based on:
        % - 4 original pixels (source_4pix)
        % - window dimmensions (SF,winW,winH,)
        % - window position (left_source_row, upper_source_row, next_x(target_x), next_y(target_y))
        
        if strcmp('int',type)
            
        output_image(target_y,target_x) = floor(winscale_4pix(source_4pix,left_source_row,upper_source_row,winW, winH,sourW,sourH,SF,next_x(target_x),next_y(target_y),scaletype));
        else
            if strcmp('float',type)
        output_image(target_y,target_x) = winscale_4pix(source_4pix,left_source_row,upper_source_row,winW, winH,sourW,sourH,SF,next_x(target_x),next_y(target_y),scaletype);
            else
                disp('No TYPE (int or float) assigned');
           end
            
        end
        
       
        
        
    end
    %
    
end

end
end

if strcmp('int',type)
    target_image = uint8(output_image);
else
    if strcmp('float',type)
        target_image = output_image;
    else
        disp('No TYPE (int or float) assigned');
    end
    
end






end

